"use strict";

var orderMap,
    geocoder,
    directionsRequest,
    directionsDisplay,
    mapDrawingManager,
    acceptanceRegionSelectedShape,
    originInputSelector,
    destinationInputSelector,
    mapGeometry,
    originalPolygon,
    mapNewShape,
    originMarker,
    destinationMarker,
    canDragOriginMarker,
    canDragDestinationMarker,
    startingPosition = { lat: 51.4965971, lng: -0.1737877 },
    polygonOptions = {
      fillColor: '#1080f2',
      fillOpacity: .4,
      strokeWeight: 5,
      clickable: true,
      editable: true,
      zIndex: 1
    },
    startingMap,
    endingMap,
    pickUpSelectedShape,
    dropOffSelectedShape,
    puOriginalPolygon,
    doOriginalPolygon,
    pickUpNewShape,
    dropOffNewShape,
    pickUpDrawingManager,
    dropOffDrawingManager,
    pickUpLatLngBounds,
    dropOffLatLngBounds,
    unitSystem,
    waypointMarkers = [];

function initMap() {
  // Custom function to get polygon bounds as google maps doesn't support it
  google.maps.Polygon.prototype.getBounds = function() {
    var bounds = new google.maps.LatLngBounds();
    var paths = this.getPaths();
    var path;
    for (var i = 0; i < paths.getLength(); i++) {
      path = paths.getAt(i);

      for (var j = 0; j < path.getLength(); j++) {
        bounds.extend(path.getAt(j));
      }
    }

    return bounds;
  }

  if ( quickcab_js_settings.uses_miles ) {
    unitSystem = google.maps.UnitSystem.IMPERIAL;
  } else {
    unitSystem = google.maps.UnitSystem.METRIC;
  }

  var directionsService = new google.maps.DirectionsService();

  if ( jQuery('#quickcab_order_map').length > 0 ) {
    orderMap = new google.maps.Map(document.getElementById('quickcab_order_map'), {
      center: startingPosition,
      zoom: 5
    });

    directionsDisplay = new google.maps.DirectionsRenderer({
      map: orderMap,
      suppressMarkers: true,
      preserveViewport: true
    });

    var originLatLng = {
      lat: parseFloat(quickcab_js_settings.order_details.starting_coords.lat),
      lng: parseFloat(quickcab_js_settings.order_details.starting_coords.long)
    };

    var destinationLatLng = {
      lat: parseFloat(quickcab_js_settings.order_details.destination_coords.lat),
      lng: parseFloat(quickcab_js_settings.order_details.destination_coords.long)
    };

    originMarker = new google.maps.Marker({
      position: originLatLng,
      map: orderMap,
      label: '1',
      title: quickcab_js_settings.order_details.starting_address
    });

    destinationMarker = new google.maps.Marker({
      position: destinationLatLng,
      map: orderMap,
      label: (quickcab_js_settings.order_details.waypoints.length + 2).toString(),
      title: quickcab_js_settings.order_details.destination_address
    });

    for ( var i = 0; i < quickcab_js_settings.order_details.waypoints.length; i++ ) {
      var waypoint = quickcab_js_settings.order_details.waypoints[i];

      waypointMarkers.push(new google.maps.Marker({
        position: new google.maps.LatLng(waypoint.lat, waypoint.lng),
        map: orderMap,
        label: (i + 2).toString(),
        title: quickcab_js_settings.order_details.destination_address
      }));
    }

    var waypoints = quickcab_js_settings.order_details.waypoints.filter(function(waypoint) {
      return waypoint.lat && waypoint.lng && waypoint.address;
    }).map(function(waypoint) {
      return {
        location: new google.maps.LatLng(waypoint.lat, waypoint.lng),
        stopover: true
      }
    });

    directionsRequest = {
      travelMode: google.maps.TravelMode.DRIVING,
      unitSystem: unitSystem,
      origin: originLatLng,
      destination: destinationLatLng,
      waypoints: waypoints
    };

    // Pass the directions request to the directions service.
    directionsService.route(directionsRequest, function(response, status) {
      if (status == google.maps.DirectionsStatus.OK) {
        directionsDisplay.setMap(orderMap);
        directionsDisplay.setDirections(response);

        google.maps.event.trigger(orderMap, 'resize');
        orderMap.setCenter(response.routes[0].bounds.getCenter());
        orderMap.fitBounds(response.routes[0].bounds);
      }
    });
  }

  var autocompleteSettingsFields = document.getElementsByClassName('quickcab-google-autocomplete-field');

  for (var i = 0; i < autocompleteSettingsFields.length; i++) {
    var settingsField = autocompleteSettingsFields[i];

    var autocomplete = new google.maps.places.Autocomplete(settingsField);

    google.maps.event.addListener(autocomplete, 'place_changed', function() {
      jQuery(settingsField).siblings('.quickcab-autocomplete-tick').addClass('-green');

      var addressValue = jQuery(settingsField).val();
      jQuery(settingsField).siblings('.quickcab-hidden-autocomplete-field').val(addressValue);
    });

    jQuery(settingsField).on('input', function() {
      jQuery(this).siblings('.quickcab-autocomplete-tick').removeClass('-green');
      jQuery(this).siblings('.quickcab-hidden-autocomplete-field').val('');
    });
  }

  // starting map
  if ( jQuery('#quickcab_geofence_starting_map').length > 0 ) {
    // Needs to be declared here as google.maps.drawing needs to be defined
    var drawingManagerOptions = {
      drawingMode: google.maps.drawing.OverlayType.POLYGON,
      drawingControl: true,
      drawingControlOptions: {
        position: google.maps.ControlPosition.TOP_CENTER,
        drawingModes: [
          google.maps.drawing.OverlayType.POLYGON
        ]
      },
      polygonOptions: polygonOptions
    };

    startingMap = new google.maps.Map(document.getElementById('quickcab_geofence_starting_map'), {
      center: startingPosition,
      zoom: 4
    });

    endingMap = new google.maps.Map(document.getElementById('quickcab_geofence_destination_map'), {
      center: startingPosition,
      zoom: 4
    });

    pickUpDrawingManager = new google.maps.drawing.DrawingManager(drawingManagerOptions);
    dropOffDrawingManager = new google.maps.drawing.DrawingManager(drawingManagerOptions);

    pickUpLatLngBounds = new google.maps.LatLngBounds();
    dropOffLatLngBounds = new google.maps.LatLngBounds();

    google.maps.event.addListener(pickUpDrawingManager, 'polygoncomplete', function(e) {
      // Do not let user draw another shape
      pickUpDrawingManager.setOptions({
        drawingControl: false,
        drawingMode: null
      });

      addCoordsToInput(e, '#quickcab_geofence_starting_coords');

      updateMapZoomBoundInfo(pickUpLatLngBounds, e, startingMap, 'starting');

      // Add an event listener that selects the newly-drawn shape when the user
      // mouses down on it.
      pickUpNewShape = e;
      pickUpNewShape.type = e.type;

      google.maps.event.addListener(pickUpNewShape.getPath(), 'set_at', function() {
        addCoordsToInput(e, '#quickcab_geofence_starting_coords');
        updateMapZoomBoundInfo(pickUpLatLngBounds, e, startingMap, 'starting');
      });

      google.maps.event.addListener(pickUpNewShape.getPath(), 'insert_at', function() {
        addCoordsToInput(e, '#quickcab_geofence_starting_coords');
        updateMapZoomBoundInfo(pickUpLatLngBounds, e, startingMap, 'starting');
      });

      pickUpSetSelection(pickUpNewShape);

      jQuery('#quickcab_geofence_starting_save_shape').attr('disabled', false);
    });

    google.maps.event.addListener(dropOffDrawingManager, 'polygoncomplete', function(e) {
      // Do not let user draw another shape
      dropOffDrawingManager.setOptions({
        drawingControl: false,
        drawingMode: null
      });

      addCoordsToInput(e, '#quickcab_geofence_destination_coords');

      updateMapZoomBoundInfo(dropOffLatLngBounds, e, endingMap, 'destination');

      // Add an event listener that selects the newly-drawn shape when the user
      // mouses down on it.
      dropOffNewShape = e;
      dropOffNewShape.type = e.type;

      google.maps.event.addListener(dropOffNewShape.getPath(), 'set_at', function() {
        addCoordsToInput(e, '#quickcab_geofence_destination_coords');
        updateMapZoomBoundInfo(dropOffLatLngBounds, e, endingMap, 'destination');
      });

      google.maps.event.addListener(dropOffNewShape.getPath(), 'insert_at', function() {
        addCoordsToInput(e, '#quickcab_geofence_destination_coords');
        updateMapZoomBoundInfo(dropOffLatLngBounds, e, endingMap, 'destination');
      });

      dropOffSetSelection(dropOffNewShape);

      jQuery('#quickcab_geofence_destination_save_shape').attr('disabled', false);
    });

    google.maps.event.addDomListener(document.getElementById('quickcab_geofence_starting_delete_shape'), 'click', function(e) {
      if (jQuery(this).attr('disabled')) {
        return false;
      }

      e.preventDefault();
      pickUpDeleteShape();

      // Allow user to draw again when shape deleted
      pickUpDrawingManager.setOptions({
        drawingControl: true,
        drawingMode: google.maps.drawing.OverlayType.POLYGON
      });

      jQuery('#quickcab_geofence_starting_save_shape_actions').hide();
      jQuery('#quickcab_geofence_starting_save_shape').attr('disabled', true);
    });

    google.maps.event.addDomListener(document.getElementById('quickcab_geofence_destination_delete_shape'), 'click', function(e) {
      if (jQuery(this).attr('disabled')) {
        return false;
      }

      e.preventDefault();
      dropOffDeleteShape();
      jQuery('#quickcab_geofence_destination_polygon_center').val('');

      // Allow user to draw again when shape deleted
      dropOffDrawingManager.setOptions({
        drawingControl: true,
        drawingMode: google.maps.drawing.OverlayType.POLYGON
      });

      jQuery('#quickcab_geofence_destination_save_shape_actions').hide();
      jQuery('#quickcab_geofence_destination_save_shape').attr('disabled', true);
    });

    pickUpDrawingManager.setMap(startingMap);
    dropOffDrawingManager.setMap(endingMap);

    var mapStartingArea = quickcab_js_settings.map_starting_area

    geocoder = new google.maps.Geocoder();

    if ( quickcab_js_settings.on_geofence_edit_page ) {
      if ( jQuery('#quickcab_geofence_starting_coords').val().length > 0 ) {
        var pu_coords = jQuery('#quickcab_geofence_starting_coords').val().split('|');
      } else {
        var pu_coords = [];
      }

      if ( jQuery('#quickcab_geofence_destination_coords').val().length > 0 ) {
        var do_coords = jQuery('#quickcab_geofence_destination_coords').val().split('|');
      } else {
        var do_coords = [];
      }

      var puFormattedCoords = [];
      var doFormattedCoords = [];

      for (var i = 0; i < pu_coords.length; i++) {
        puFormattedCoords.push({
          lat: parseFloat(pu_coords[i].split(',')[0]),
          lng: parseFloat(pu_coords[i].split(',')[1])
        });
      }

      for (var i = 0; i < do_coords.length; i++) {
        doFormattedCoords.push({
          lat: parseFloat(do_coords[i].split(',')[0]),
          lng: parseFloat(do_coords[i].split(',')[1])
        });
      }

      // JSON.parse(JSON.stringify()) done to ensure new object is not a
      // reference to polygonOptions
      var pickUpPolygonOptions = JSON.parse(JSON.stringify(polygonOptions));
      var dropOffPolygonOptions = JSON.parse(JSON.stringify(polygonOptions));

      pickUpPolygonOptions['paths'] = puFormattedCoords;
      dropOffPolygonOptions['paths'] = doFormattedCoords;

      puOriginalPolygon = new google.maps.Polygon(pickUpPolygonOptions);
      doOriginalPolygon = new google.maps.Polygon(dropOffPolygonOptions);

      puOriginalPolygon.setMap(startingMap);
      doOriginalPolygon.setMap(endingMap);

      if ( puOriginalPolygon.getPath() ) {
        startingMap.fitBounds(puOriginalPolygon.getBounds());

        // When a vertex of the polygon is moved
        google.maps.event.addListener(puOriginalPolygon.getPath(), 'set_at', function(e) {
          addCoordsToInput(puOriginalPolygon, '#quickcab_geofence_starting_coords');
          updateMapZoomBoundInfo(pickUpLatLngBounds, puOriginalPolygon, startingMap, 'starting');
        });

        // When a new vertex is added to the polygon
        google.maps.event.addListener(puOriginalPolygon.getPath(), 'insert_at', function(e) {
          addCoordsToInput(puOriginalPolygon, '#quickcab_geofence_starting_coords');
          updateMapZoomBoundInfo(pickUpLatLngBounds, puOriginalPolygon, startingMap, 'starting');
        });

        pickUpSetSelection(puOriginalPolygon);
        addCoordsToInput(puOriginalPolygon, '#quickcab_geofence_starting_coords');

        pickUpDrawingManager.setOptions({
          drawingControl: false,
          drawingMode: null
        });
      } else if (mapStartingArea) {
        geocoder.geocode({ 'address': mapStartingArea }, function(results, status) {
          if (status == google.maps.GeocoderStatus.OK ) {
            startingMap.setCenter(results[0].geometry.location);
            startingMap.fitBounds(results[0].geometry.viewport);
          }
        });
      }

      if ( doOriginalPolygon.getPath() ) {
        endingMap.fitBounds(doOriginalPolygon.getBounds());

        // When a vertex of the polygon is moved
        google.maps.event.addListener(doOriginalPolygon.getPath(), 'set_at', function(e) {
          addCoordsToInput(doOriginalPolygon, '#quickcab_geofence_destination_coords');
          updateMapZoomBoundInfo(dropOffLatLngBounds, doOriginalPolygon, endingMap, 'ending');
        });

        // When a new vertex is added to the polygon
        google.maps.event.addListener(doOriginalPolygon.getPath(), 'insert_at', function(e) {
          addCoordsToInput(doOriginalPolygon, '#quickcab_geofence_destination_coords');
          updateMapZoomBoundInfo(dropOffLatLngBounds, doOriginalPolygon, endingMap, 'ending');
        });

        dropOffSetSelection(doOriginalPolygon);
        addCoordsToInput(doOriginalPolygon, '#quickcab_geofence_destination_coords');

        dropOffDrawingManager.setOptions({
          drawingControl: false,
          drawingMode: null
        });
      } else if (mapStartingArea) {
        geocoder.geocode({ 'address': mapStartingArea }, function(results, status) {
          if (status == google.maps.GeocoderStatus.OK ) {
            endingMap.setCenter(results[0].geometry.location);
            endingMap.fitBounds(results[0].geometry.viewport);
          }
        });
      }
    } else if (quickcab_js_settings.on_geofence_new_page) {
      geocoder.geocode({ 'address': mapStartingArea }, function(results, status) {
        if (status == google.maps.GeocoderStatus.OK ) {
          startingMap.setCenter(results[0].geometry.location);
          startingMap.fitBounds(results[0].geometry.viewport);

          endingMap.setCenter(results[0].geometry.location);
          endingMap.fitBounds(results[0].geometry.viewport);
        }
      });
    }
  }

  jQuery('#quickcab_geofence_starting_import_shape_button').on('click', function(e) {
    e.preventDefault();

    var newCoords = jQuery('#quickcab_geofence_starting_shape option:selected').attr('data-coordinates');
    jQuery('#quickcab_geofence_starting_coords').val(newCoords);

    if ( puOriginalPolygon ) {
      puOriginalPolygon.setMap(null);
    }

    var puFormattedCoords = [];
    var pu_coords = newCoords.split('|');

    for (var i = 0; i < pu_coords.length; i++) {
      puFormattedCoords.push({
        lat: parseFloat(pu_coords[i].split(',')[0]),
        lng: parseFloat(pu_coords[i].split(',')[1])
      });
    }

    var pickUpPolygonOptions = JSON.parse(JSON.stringify(polygonOptions));
    pickUpPolygonOptions['paths'] = puFormattedCoords;
    puOriginalPolygon = new google.maps.Polygon(pickUpPolygonOptions);
    puOriginalPolygon.setMap(startingMap);
    startingMap.fitBounds(puOriginalPolygon.getBounds());
    google.maps.event.trigger(pickUpDrawingManager, 'polygoncomplete', puOriginalPolygon);
  });

  jQuery('#quickcab_geofence_destination_import_shape_button').on('click', function(e) {
    e.preventDefault();

    var newCoords = jQuery('#quickcab_geofence_destination_shape option:selected').attr('data-coordinates');
    jQuery('#quickcab_geofence_destination_coords').val(newCoords);

    if ( doOriginalPolygon ) {
      doOriginalPolygon.setMap(null);
    }

    var doFormattedCoords = [];
    var do_coords = newCoords.split('|');

    for (var i = 0; i < do_coords.length; i++) {
      doFormattedCoords.push({
        lat: parseFloat(do_coords[i].split(',')[0]),
        lng: parseFloat(do_coords[i].split(',')[1])
      });
    }

    var dropOffPolygonOptions = JSON.parse(JSON.stringify(polygonOptions));
    dropOffPolygonOptions['paths'] = doFormattedCoords;
    doOriginalPolygon = new google.maps.Polygon(dropOffPolygonOptions);
    doOriginalPolygon.setMap(endingMap);
    endingMap.fitBounds(doOriginalPolygon.getBounds());
    google.maps.event.trigger(dropOffDrawingManager, 'polygoncomplete', doOriginalPolygon);
  });
}

function acceptanceRegionClearSelection() {
  if (acceptanceRegionSelectedShape) {
    acceptanceRegionSelectedShape.setEditable(false);
    acceptanceRegionSelectedShape = null;
  }
}

function acceptanceRegionSetSelection(shape) {
  acceptanceRegionClearSelection();
  acceptanceRegionSelectedShape = shape;
  shape.setEditable(true);
}

function acceptanceRegionDeleteSelectedShape() {
  if (acceptanceRegionSelectedShape) {
    acceptanceRegionSelectedShape.setMap(null);
  }

  jQuery('.quickcab__setting--acceptance-region').val('');
}

function addCoordsToInput(e, identifier) {
  var coords = '';
  for (var i = 0; i < e.getPath().getLength(); i++) {
    coords += e.getPath().getAt(i).toUrlValue(8) + '|';
  }
  coords = coords.slice(0, -1);

  jQuery(identifier).val(coords);
}


function triggerDrawingMapResize(map, polygon, selector) {
  google.maps.event.trigger(map, "resize");

  if ( jQuery(selector).val().length > 0 ) {
    map.fitBounds(polygon.getBounds());
  } else {
    map.setCenter(mapGeometry.location);
    map.fitBounds(mapGeometry.viewport);
  }
}

function toggleAcceptanceAreaMap(value) {
  if (value == '1') {
    jQuery('.acceptance-region-map').slideDown(400, function() {
      triggerDrawingMapResize(map, originalPolygon, '.quickcab__setting--acceptance-region');
    });
  } else {
    jQuery('.acceptance-region-map').slideUp();
  }
}

function quickcabGetParameterByName(name) {
  name = name.replace(/[\[\]]/g, "\\$&");

  var url = window.location.href;
  var regex = new RegExp("[?&]" + name + "(=([^&#]*)|&|#|$)");
  var results = regex.exec(url);

  if ( !results ) {
    return null;
  }

  if ( !results[2] ) {
    return '';
  }

  return decodeURIComponent(results[2].replace(/\+/g, " "));
}

function onlyUnique(value, index, self) {
  return self.indexOf(value) === index;
}

jQuery('.quickcab__setting--enforce-rejection-region').change(function() {
  toggleAcceptanceAreaMap(this.value);
});

jQuery('.quickcab__settings--maximum-fare.edit').keypress(function(e) {
  var charCode = (e.which) ? e.which : e.keyCode;
  // If key pressed isn't decimal point or number
  if (charCode != 46 && charCode > 31 && (charCode < 48 || charCode > 57)) {
    return false;
  }

  // If input already has a decimal place
  if ( charCode == 46 && this.value.indexOf('.') > -1 ) {
    return false;
  }

  return true;
});

jQuery('.quickcab__settings--minimum-fare.edit').keypress(function(e) {
  var charCode = (e.which) ? e.which : e.keyCode;
  // If key pressed isn't decimal point or number
  if (charCode != 46 && charCode > 31 && (charCode < 48 || charCode > 57)) {
    return false;
  }

  // If input already has a decimal place
  if ( charCode == 46 && this.value.indexOf('.') > -1 ) {
    return false;
  }

  return true;
});

jQuery('.quickcab__settings--enforce-maximum-fare').change(function() {
  if (this.value == '1') {
    jQuery('.quickcab__settings--maximum-fare').attr('disabled', false);
    jQuery('#QuickCabMaximumFareContainer').show();
  } else {
    jQuery('.quickcab__settings--maximum-fare').attr('disabled', 'disabled');
    jQuery('#QuickCabMaximumFareContainer').hide();
  }
});

jQuery('.quickcab__settings--enforce-minimum-fare').change(function() {
  if (this.value == '1') {
    jQuery('.quickcab__settings--minimum-fare').attr('disabled', false);
    jQuery('#QuickCabMinimumFareContainer').show();
  } else {
    jQuery('.quickcab__settings--minimum-fare').attr('disabled', 'disabled');
    jQuery('#QuickCabMinimumFareContainer').hide();
  }
});

jQuery('.quickcab__setting--enforce-minimum-distance').change(function() {
  if (this.value == '1') {
    jQuery('.quickcab__setting--minimum-distance').attr('disabled', false);
    jQuery('.quickcab__setting--minimum-distance-error-message').attr('disabled', false);
    jQuery('#QuickCabMinimumDistanceContainer').show();
    jQuery('#QuickCabMinimumDistanceErrorContainer').show();
  } else {
    jQuery('.quickcab__setting--minimum-distance').attr('disabled', 'disabled');
    jQuery('.quickcab__setting--minimum-distance-error-message').attr('disabled', 'disabled');
    jQuery('#QuickCabMinimumDistanceContainer').hide();
    jQuery('#QuickCabMinimumDistanceErrorContainer').hide();
  }
});

jQuery('.quickcab__setting--enforce-maximum-distance').change(function() {
  if (this.value == '1') {
    jQuery('.quickcab__setting--maximum-distance').attr('disabled', false);
    jQuery('.quickcab__setting--maximum-distance-error-message').attr('disabled', false);
    jQuery('#QuickCabMaximumDistanceContainer').show();
    jQuery('#QuickCabMaximumDistanceErrorContainer').show();
  } else {
    jQuery('.quickcab__setting--maximum-distance').attr('disabled', 'disabled');
    jQuery('.quickcab__setting--maximum-distance-error-message').attr('disabled', 'disabled');
    jQuery('#QuickCabMaximumDistanceContainer').hide();
    jQuery('#QuickCabMaximumDistanceErrorContainer').hide();
  }
});

jQuery('.quickcab__setting--title-or-image').change(function() {
  if (this.value == 'title') {
    jQuery('.quickcab__setting--form-title').attr('disabled', false);
    jQuery('.quickcab__setting--form-image-url').attr('disabled', 'disabled');
    jQuery('#QuickcabFormTitleContainer').show();
    jQuery('#QuickcabFormTitleImageContainer').hide();
  } else {
    jQuery('.quickcab__setting--form-title').attr('disabled', 'disabled');
    jQuery('.quickcab__setting--form-image-url').attr('disabled', false);
    jQuery('#QuickcabFormTitleContainer').hide();
    jQuery('#QuickcabFormTitleImageContainer').show();
  }
});

jQuery('.quickcab__setting--can-edit-vehicle-type').change(function() {
  if (this.value == '1') {
    jQuery('.quickcab__setting--default-vehicle-id').attr('disabled', 'disabled');
    jQuery('#QuickCabDefaultVehicleContainer').hide();
  } else {
    jQuery('.quickcab__setting--default-vehicle-id').attr('disabled', false);
    jQuery('#QuickCabDefaultVehicleContainer').show();
  }
});

jQuery('.quickcab__setting--enforce-rejection-region').change(function() {
  if (this.value == '1') {
    jQuery('.quickcab__setting--acceptance-region-error-message').attr('disabled', false);
    jQuery('#QuickCabAcceptanceRegionErrorContainer').show();
  } else {
    jQuery('.quickcab__setting--acceptance-region-error-message').attr('disabled', 'disabled');
    jQuery('#QuickCabAcceptanceRegionErrorContainer').hide();
  }
});

jQuery('.quickcab__settings--enforce-autocomplete-restriction').change(function() {
  if (this.value == '1') {
    jQuery('.quickcab__settings--autocomplete-country').attr('disabled', false);
    jQuery('#QuickCabAutocompleteCountryRestrictionContainer').show();
  } else {
    jQuery('.quickcab__settings--autocomplete-country').attr('disabled', 'disabled');
    jQuery('#QuickCabAutocompleteCountryRestrictionContainer').hide();
  }
});

// Make textareas grow when return pressed
jQuery('textarea').css('overflow', 'hidden');
jQuery("textarea").keyup(function(e) {
  while(jQuery(this).outerHeight() < this.scrollHeight + parseFloat(jQuery(this).css("borderTopWidth")) + parseFloat(jQuery(this).css("borderBottomWidth"))) {
    jQuery(this).height(jQuery(this).height()+1);
  };
});

jQuery('.quickcab__new--vat-all-day').change(function() {
  if ( this.value == '1' ) {
    jQuery('.quickcab__new--vat-start-time.' + jQuery(this).attr('data-day')).attr('disabled', 'disabled');
    jQuery('.quickcab__new--vat-end-time.' + jQuery(this).attr('data-day')).attr('disabled', 'disabled');
  } else {
    jQuery('.quickcab__new--vat-start-time.' + jQuery(this).attr('data-day')).attr('disabled', false);
    jQuery('.quickcab__new--vat-end-time.' + jQuery(this).attr('data-day')).attr('disabled', false);
  }
});

jQuery(document).ready(function() {
  jQuery('.quickcab-select').select2();
});

/**
 * New Post
 */
jQuery('#quickcab_vehicle_pricing_structure').change(function() {
  jQuery('#quickcab_modular_pricing_options').hide();
  jQuery('#quickcab_modular_pricing_options_part_2').hide();
  jQuery('#quickcab_modular_pricing_fixed_options').hide();
  jQuery('#quickcab_modular_pricing_tiered_options').hide();
  jQuery('#quickcab_hourly_pricing_options').hide();
  jQuery('#quickcab_hourly_pricing_tiered_options').hide();
  jQuery('#quickcab_vehicle_price_per_distance_helper_text').hide();
  jQuery('#quickcab_vehicle_price_per_hour_helper_text').hide();

  switch (this.value) {
    case 'per-distance':
      jQuery('#quickcab_modular_pricing_options').show();
      jQuery('#quickcab_modular_pricing_fixed_options').show();
      jQuery('#quickcab_modular_pricing_options_part_2').show();
      break;
    case 'per-distance-tiered':
      jQuery('#quickcab_modular_pricing_options').show();
      jQuery('#quickcab_modular_pricing_fixed_options').show();
      jQuery('#quickcab_modular_pricing_tiered_options').show();
      jQuery('#quickcab_modular_pricing_options_part_2').show();
      jQuery('#quickcab_vehicle_price_per_distance_helper_text').show();
      break;
    case 'per-hour':
      jQuery('#quickcab_hourly_pricing_options').show();
      break;
    case 'per-hour-tiered':
      jQuery('#quickcab_hourly_pricing_options').show();
      jQuery('#quickcab_hourly_pricing_tiered_options').show();
      jQuery('#quickcab_vehicle_price_per_hour_helper_text').show();
      break;
  }
});

jQuery('body').on('click', '.qc-pricing-tier-remove', function(e) {
  e.preventDefault();
  var type = jQuery(this).attr('data-tiered-type');
  jQuery(this).closest('tr').remove();
  quickcabRecalculateTieredValues(jQuery('.quickcab-tiered-table.-' + type));
});

jQuery('body').on('focusout', '.quickcab-tiered-pricing-value-to', function(e) {
  var type = jQuery(this).attr('data-tiered-type');
  quickcabRecalculateTieredValues(jQuery('.quickcab-tiered-table.-' + type));
});

jQuery('.quickcab-color-picker').spectrum({
  preferredFormat: 'hex3',
  showInput: true,
  showPalette: true,
  palette: [
    ['#1080f2', '#9b59b6', '#e84118'],
    ['#f0932b', '#f1c40f', '#4cd137'],
    ['#48dbfb', '#fff', '#000'],
    [
      quickcab_js_settings.form.form_primary_color,
      quickcab_js_settings.form.form_secondary_color
    ]
  ]
});

jQuery('.quickcab-order-change-customer').on('click', function(e) {
  e.preventDefault();

  jQuery('.quickcab-order-customer-info-container').hide();
  jQuery('.quickcab-order-change-customer-cont').show();
  jQuery('.quickcab-order-change-customer-cancel').show();
  jQuery('.quickcab-order-change-customer').hide();
  jQuery('.quickcab-select').select2();
});

jQuery('.quickcab-order-change-customer-cancel').on('click', function(e) {
  e.preventDefault();

  jQuery('.quickcab-order-customer-info-container').show();
  jQuery('.quickcab-order-change-customer-cont').hide();
  jQuery('.quickcab-order-change-customer-cancel').hide();
  jQuery('.quickcab-order-change-customer').show();
  jQuery('#quickcab_order_customer_id').val([]);
});

function pickUpSetSelection(shape) {
  pickUpSelectedShape = shape;
  shape.setEditable(true);
}

function dropOffSetSelection(shape) {
  dropOffSelectedShape = shape;
  shape.setEditable(true);
}

function pickUpDeleteShape() {
  if ( pickUpSelectedShape ) {
    pickUpSelectedShape.setMap(null);
  }

  jQuery('#quickcab_geofence_starting_coords').val('');
}

function dropOffDeleteShape() {
  if ( dropOffSelectedShape ) {
    dropOffSelectedShape.setMap(null);
  }

  jQuery('#quickcab_geofence_destination_coords').val('');
}

function addCoordsToInput(e, identifier) {
  var coords = '';
  for ( var i = 0; i < e.getPath().getLength(); i++ ) {
    coords += e.getPath().getAt(i).toUrlValue(8) + '|';
  }
  coords = coords.slice(0, -1);

  jQuery(identifier).val(coords);
}

/**
 * Update zoom/centre info for given shape
 */
function updateMapZoomBoundInfo(latLngBounds, e, map, startingEnding) {
  latLngBounds = new google.maps.LatLngBounds();

  // Add all paths to bounds
  for ( var i = 0; i < e.getPath().getLength(); i++ ) {
    latLngBounds.extend(e.getPath().getAt(i));
  }

  // Save map centre for image used on index page
  var formattedCenter = latLngBounds.getCenter().lat() + ',' + latLngBounds.getCenter().lng();
  jQuery('#quickcab_geofence_' + startingEnding + '_polygon_center').val(formattedCenter);

  var zoom = getBoundsZoomLevel(latLngBounds, { height: 300, width: 300 });

  jQuery('#quickcab_geofence_' + startingEnding + '_polygon_zoom').val(zoom);
}

/**
 * Calculates the zoom required for a map to fit the current points
 *
 * @since  1.0
 * @see    https://stackoverflow.com/questions/6048975/google-maps-v3-how-to-calculate-the-zoom-level-for-a-given-bounds
 * @param  object   bounds  the bounds (google maps)
 * @param  object   mapDim  map dimension
 * @return int              desired zoom level
 */
function getBoundsZoomLevel(bounds, mapDim) {
  var WORLD_DIM = { height: 256, width: 256 };
  var ZOOM_MAX = 21;

  function latRad(lat) {
    var sin = Math.sin(lat * Math.PI / 180);
    var radX2 = Math.log((1 + sin) / (1 - sin)) / 2;
    return Math.max(Math.min(radX2, Math.PI), -Math.PI) / 2;
  }

  function zoom(mapPx, worldPx, fraction) {
    return Math.floor(Math.log(mapPx / worldPx / fraction) / Math.LN2);
  }

  var ne = bounds.getNorthEast();
  var sw = bounds.getSouthWest();

  var latFraction = (latRad(ne.lat()) - latRad(sw.lat())) / Math.PI;

  var lngDiff = ne.lng() - sw.lng();
  var lngFraction = ((lngDiff < 0) ? (lngDiff + 360) : lngDiff) / 360;

  var latZoom = zoom(mapDim.height, WORLD_DIM.height, latFraction);
  var lngZoom = zoom(mapDim.width, WORLD_DIM.width, lngFraction);

  return Math.min(latZoom, lngZoom, ZOOM_MAX);
}

function quickcabShowNotice(text, color) {
  jQuery('.quickcab-notice').text(text).slideDown().addClass('quickcab-notice-' + color);

  setTimeout(function() {
    jQuery('.quickcab-notice').text('').slideUp().removeClass('quickcab-notice-' + color);
  }, 5000);
}

jQuery('#quickcab_geofence_pricing_structure').change(function() {
  jQuery('#quickcab_geofence_fixed_price').hide();
  jQuery('#quickcab_geofence_increased_price').hide();
  jQuery('#quickcab_geofence_ignore_surge_pricing_rule_cont').hide();

  switch (this.value) {
    case 'fixed-price':
      jQuery('#quickcab_geofence_fixed_price').show();
      jQuery('#quickcab_geofence_ignore_surge_pricing_rule_cont').show();
      break;
    case 'increased-price':
      jQuery('#quickcab_geofence_increased_price').show();
      break;
  }
});

jQuery('#quickcab_geofence_vehicle_0').change(function() {
  var allChecked = this.checked;

  jQuery('.quickcab__geofence--vehicles-checkboxes').each(function() {
    this.checked = allChecked;
  });
});

jQuery('.quickcab__geofence--vehicles-checkboxes').change(function() {
  if (jQuery('#quickcab_geofence_vehicle_0').is(":checked") && !this.checked) {
    jQuery('#quickcab_geofence_vehicle_0').attr("checked", false);
  } else {
    var allChecked = true;

    jQuery('.quickcab__geofence--vehicles-checkboxes').each(function(index) {
      if (!this.checked) {
        allChecked = false;
      }
    });

    jQuery('#quickcab_geofence_vehicle_0').attr("checked", allChecked);
  }
});

jQuery('#quickcab_surge_price_pricing_structure').change(function() {
  jQuery('#quickcab_surge_price_increase_amount').hide();
  jQuery('#quickcab_surge_price_increase_multiplier').hide();

  switch (this.value) {
    case 'increase-amount':
      jQuery('#quickcab_surge_price_increase_amount').show();
      break;
    case 'increase-multiplier':
      jQuery('#quickcab_surge_price_increase_multiplier').show();
      break;
  }
});

function quickcabRecalculateTieredValues(table) {
  var prevRowMax = 0;
  var incrementAmount = wpdk_apply_filters('quickcab_tiered_pricing_field_increment_amount', 1);

  table.find('tr').each(function(i, el) {
    if (i == 0) {
      return;
    }

    var row = jQuery(el);
    var firstInput = row.find('td:eq(0) input');
    var secondInput = row.find('td:eq(1) input');

    if (i == 1) {
      firstInput.val('0');
      firstInput.attr('min', '0');
      firstInput.attr('disabled', true);
    } else {
      firstInput.attr('min', prevRowMax);

      if (
        (firstInput.val() == '' || parseFloat(firstInput.val()) != prevRowMax + incrementAmount)
        &&
        prevRowMax
      ) {
        firstInput.val(prevRowMax + incrementAmount);
      }
    }

    if ( parseFloat(secondInput.val()) <= parseFloat(firstInput.val()) ) {
      secondInput.val(parseFloat(firstInput.val()) + incrementAmount);
    }

    prevRowMax = parseFloat(secondInput.val())
  });
}

jQuery('#tiered_pricing_distance_add_tier').on('click', function(e) {
  e.preventDefault();
  var order = jQuery('.quickcab-tiered-table.-distance tbody tr').length;
  var actions = wpdk_apply_filters('quickcab_tiered_pricing_distance_actions', '<a href="#" class="button qc-pricing-tier-remove" data-tiered-type="distance">' + quickcab_js_settings.translations.remove_tier + '</a>', order);
  var price_suffix = wpdk_apply_filters('quickcab_tiered_pricing_distance_price_suffix', '');

  var elementText = '<tr>\
    <td>\
      <input type="number" min="0" step="0.1" class="small-text quickcab-tiered-pricing-value-from" name="quickcab_vehicle_price_per_distance_tiered[' + order + '][value_from]">\
    </td>\
    <td>\
      <input type="number" min="0" step="0.1" class="small-text quickcab-tiered-pricing-value-to" name="quickcab_vehicle_price_per_distance_tiered[' + order + '][value_to]" data-tiered-type="distance">\
    </td>\
    <td>\
      ' + quickcab_js_settings.currency_symbol + '<input type="number" min="0" step="0.1" class="small-text" name="quickcab_vehicle_price_per_distance_tiered[' + order + '][price]">' + price_suffix + '\
    </td>\
    <td>' + actions + '</td>\
  </tr>';

  jQuery(elementText).appendTo('.quickcab-tiered-table.-distance tbody');

  quickcabRecalculateTieredValues(jQuery('.quickcab-tiered-table.-distance'))
});

jQuery('#tiered_pricing_hourly_add_tier').on('click', function(e) {
  e.preventDefault();
  var order = jQuery('.quickcab-tiered-table.-hourly tbody tr').length;

  var elementText = '<tr>\
    <td>\
      <input type="number" min="0" step="0.1" class="small-text quickcab-tiered-pricing-value-from" name="quickcab_vehicle_price_per_hour_tiered[' + order + '][value_from]">\
    </td>\
    <td>\
      <input type="number" min="0" step="0.1" class="small-text quickcab-tiered-pricing-value-to" name="quickcab_vehicle_price_per_hour_tiered[' + order + '][value_to]" data-tiered-type="hourly">\
    </td>\
    <td>\
      ' + quickcab_js_settings.currency_symbol + '<input type="number" min="0" step="0.01" class="small-text" name="quickcab_vehicle_price_per_hour_tiered[' + order + '][price]">\
    </td>\
    <td>\
      <a href="#" class="button qc-pricing-tier-remove" data-tiered-type="hourly">' + quickcab_js_settings.translations.remove_tier + '</a>\
    </td>\
  </tr>';

  jQuery(elementText).appendTo('.quickcab-tiered-table.-hourly tbody');

  quickcabRecalculateTieredValues(jQuery('.quickcab-tiered-table.-hourly'))
});

jQuery('#quickcab_geofence_starting_save_shape').on('click', function (e) {
  e.preventDefault();

  if ( !jQuery(this).attr('disabled') ) {
    jQuery('#quickcab_geofence_starting_import_shape_actions').hide();
    jQuery('#quickcab_geofence_starting_save_shape_actions').slideDown();
  }
});

jQuery('#quickcab_geofence_destination_save_shape').on('click', function (e) {
  e.preventDefault();

  if ( !jQuery(this).attr('disabled') ) {
    jQuery('#quickcab_geofence_destination_import_shape_actions').hide();
    jQuery('#quickcab_geofence_destination_save_shape_actions').slideDown();
  }
});

function quickcabUpdateShapeLists(shapes, message, colour) {
  var htmlString = '';
  var optionString = '';

  // populate geofence shapes sidebar metabox
  for (var i = 0; i < shapes.length; i++) {
    htmlString += '<li>' + shapes[i]['\0*\0name'] + ' <a href="#" class="quickcab-delete-geofence-shape text-red" data-geofence-shape-id="' + shapes[i]['\0*\0id'] + '">(' + quickcab_js_settings.translations.delete + ')</a></li>';
  }

  // populate list of shapes to import
  for (var i = 0; i < shapes.length; i++) {
    optionString += '<option value="' + shapes[i]['\0*\0id'] + '" data-coordinates="' + shapes[i]['serialised_coordinates'] + '">' + shapes[i]['\0*\0name'] + '</option>';
  }

  optionString = '<option disabled selected>' + quickcab_js_settings.translations.geofence.select_shape +'...</option>' + optionString;

  jQuery('.quickcab-geofence-metabox-shape-list').html(htmlString);

  jQuery('#quickcab_geofence_starting_shape').html(optionString);
  jQuery('#quickcab_geofence_destination_shape').html(optionString);

  quickcabShowNotice(message, colour);
}

jQuery('#quickcab_geofence_starting_save_shape_button').on('click', function (e) {
  e.preventDefault();

  if ( jQuery('#quickcab_geofence_starting_coords').val().length == 0 ) {
    quickcabShowNotice(quickcab_js_settings.translations.geofence.no_shape_on_map, 'red');
  } else {
    jQuery.ajax({
      type: 'post',
      url: quickcab_js_settings.ajax_url,
      data: {
        action: 'quickcab_create_geofence_shape',
        geofence_shape_name: jQuery('#quickcab_geofence_starting_shape_name').val(),
        geofence_shape_coords: jQuery('#quickcab_geofence_starting_coords').val()
      },
      success: function(return_data) {
        var parsed_data = JSON.parse(return_data);
        quickcabUpdateShapeLists(parsed_data.shapes, quickcab_js_settings.translations.geofence.shape_saved, 'green');
      }
    });
  }
});

jQuery('#quickcab_geofence_destination_save_shape_button').on('click', function(e) {
  e.preventDefault();

  if ( jQuery('#quickcab_geofence_destination_coords').val().length == 0 ) {
    quickcabShowNotice(quickcab_js_settings.translations.geofence.no_shape_on_map, 'red');
  } else {
    jQuery.ajax({
      type: 'post',
      url: quickcab_js_settings.ajax_url,
      data: {
        action: 'quickcab_create_geofence_shape',
        geofence_shape_name: jQuery('#quickcab_geofence_destination_shape_name').val(),
        geofence_shape_coords: jQuery('#quickcab_geofence_destination_coords').val()
      },
      success: function(return_data) {
        var parsed_data = JSON.parse(return_data);
        quickcabUpdateShapeLists(parsed_data.shapes, quickcab_js_settings.translations.geofence.shape_saved, 'green');
      }
    });
  }
});

jQuery('#quickcab_geofence_starting_import_shape').on('click', function(e) {
  e.preventDefault();

  jQuery('#quickcab_geofence_starting_save_shape_actions').hide();
  jQuery('#quickcab_geofence_starting_import_shape_actions').slideDown();
});

jQuery('#quickcab_geofence_destination_import_shape').on('click', function(e) {
  e.preventDefault();

  jQuery('#quickcab_geofence_destination_save_shape_actions').hide();
  jQuery('#quickcab_geofence_destination_import_shape_actions').slideDown();
});

jQuery("#quickcab_surge_price_fixed_start_date").datepicker({
  dateFormat: 'dd/mm/yy',
  showOtherMonths: true,
  dayNamesMin: ['S', 'M', 'T', 'W', 'T', 'F', 'S'],
  onSelect: function(date) {
    var splitDate = date.split('/').reverse();

    var returnMinDate = new Date(
      parseInt(splitDate[0]),
      parseInt(splitDate[1]) - 1,
      parseInt(splitDate[2]),
      0,
      0,
      0,
      0
    );

    jQuery('#quickcab_surge_price_fixed_end_date').datepicker('option', 'minDate', returnMinDate);
  }
});

jQuery("#quickcab_surge_price_fixed_end_date").datepicker({
  dateFormat: 'dd/mm/yy',
  showOtherMonths: true,
  dayNamesMin: ['S', 'M', 'T', 'W', 'T', 'F', 'S']
});

jQuery('#quickcab_surge_price_date_structure').change(function() {
  jQuery('#quickcab_surge_price_fixed_date_section').hide();
  jQuery('#quickcab_surge_price_day_of_week_section').hide();

  switch ( this.value ) {
    case 'fixed-date':
      jQuery('#quickcab_surge_price_fixed_date_section').show();
      break;
    case 'days-of-week':
      jQuery('#quickcab_surge_price_day_of_week_section').show();
      break;
  }
});

jQuery('#quickcab_surge_price_day_add_button').on('click', function(e) {
  e.preventDefault();

  var existingTiers = jQuery('#quickcab_surge_price_table_body > tr').length;

  var timeFromText = jQuery('#quickcab_surge_price_day_start_day option:selected').text() +
    ' ' + quickcab_js_settings.translations.at + ' ' +
    jQuery('#quickcab_surge_price_day_start_time_hour option:selected').text() + ':' +
    jQuery('#quickcab_surge_price_day_start_time_minute option:selected').text();
  var timeToText = jQuery('#quickcab_surge_price_day_end_day option:selected').text() +
    ' ' + quickcab_js_settings.translations.at + ' ' +
    jQuery('#quickcab_surge_price_day_end_time_hour option:selected').text() + ':' +
    jQuery('#quickcab_surge_price_day_end_time_minute option:selected').text();;
  var actions = '<a href="#" class="button button-default quickcab-surge-price__remove-time-row">' +
    quickcab_js_settings.translations.remove_time +
  '</a>';

  var fromDayVal  = jQuery('#quickcab_surge_price_day_start_day').val();
  var fromHourVal = jQuery('#quickcab_surge_price_day_start_time_hour').val();
  var fromMinVal  = jQuery('#quickcab_surge_price_day_start_time_minute').val();
  var toDayVal    = jQuery('#quickcab_surge_price_day_end_day').val();
  var toHourVal   = jQuery('#quickcab_surge_price_day_end_time_hour').val();
  var toMinVal    = jQuery('#quickcab_surge_price_day_end_time_minute').val();

  var hiddenInputs = '\
    <input type="hidden" name="quickcab_surge_price_day_tiers[' + existingTiers + '][from_day]" value="' + fromDayVal + '">\
    <input type="hidden" name="quickcab_surge_price_day_tiers[' + existingTiers + '][from_hour]" value="' + fromHourVal + '">\
    <input type="hidden" name="quickcab_surge_price_day_tiers[' + existingTiers + '][from_min]" value="' + fromMinVal + '">\
    <input type="hidden" name="quickcab_surge_price_day_tiers[' + existingTiers + '][to_day]" value="' + toDayVal + '">\
    <input type="hidden" name="quickcab_surge_price_day_tiers[' + existingTiers + '][to_hour]" value="' + toHourVal + '">\
    <input type="hidden" name="quickcab_surge_price_day_tiers[' + existingTiers + '][to_min]" value="' + toMinVal + '">';

  var newElement = '<tr>\
    <td>' + timeFromText + '</td>\
    <td>' + timeToText + '</td>\
    <td>' + actions + hiddenInputs + '</td>\
  </tr>';

  jQuery(newElement).appendTo('#quickcab_surge_price_table_body');
});

jQuery('body').on('click', '.quickcab-surge-price__remove-time-row', function(e) {
  e.preventDefault();

  jQuery(this).closest('tr').remove();
});

jQuery('input[type=radio][name=quickcab_geofence_starting_map_anywhere]').change(function() {
  if ( this.value == 'map' ) {
    jQuery('#quickcab_geofence_starting_delete_shape').attr('disabled', false);
    jQuery('#quickcab_geofence_starting_save_shape').attr('disabled', false);
    jQuery('.quickcab-geofence-map-overlay.-starting').hide();
  } else if ( this.value == 'anywhere' ) {
    jQuery('#quickcab_geofence_starting_delete_shape').attr('disabled', true);
    jQuery('#quickcab_geofence_starting_save_shape').attr('disabled', true);
    jQuery('.quickcab-geofence-map-overlay.-starting').show();
  }
});

jQuery('input[type=radio][name=quickcab_geofence_destination_map_anywhere]').change(function() {
  if ( this.value == 'map' ) {
    jQuery('#quickcab_geofence_destination_delete_shape').attr('disabled', false);
    jQuery('#quickcab_geofence_destination_save_shape').attr('disabled', false);
    jQuery('.quickcab-geofence-map-overlay.-destination').hide();
  } else if ( this.value == 'anywhere' ) {
    jQuery('#quickcab_geofence_destination_delete_shape').attr('disabled', true);
    jQuery('#quickcab_geofence_destination_save_shape').attr('disabled', true);
    jQuery('.quickcab-geofence-map-overlay.-destination').show();
  }
});

jQuery('body').on('click', '.quickcab-delete-geofence-shape', function(e) {
  if ( confirm(quickcab_js_settings.translations.geofence.delete_geofence_shape) ) {
    jQuery.ajax({
      type: 'post',
      url: quickcab_js_settings.ajax_url,
      data: {
        action: 'quickcab_delete_geofence_shape',
        geofence_shape_id: jQuery(this).attr('data-geofence-shape-id')
      },
      success: function(return_data) {
        var parsed_data = JSON.parse(return_data);

        if (parsed_data.success) {
          quickcabUpdateShapeLists(parsed_data.shapes, quickcab_js_settings.translations.geofence.shape_deleted, 'green');
        }
      }
    });
  }
});

jQuery('.quickcab-google-autocomplete-field').keypress(function(e) {
  if (e.keyCode == 13) {
    e.preventDefault();
  }
});

jQuery('#quickcab_settings\\[enforce_autocompelte_country_restriction\\]').on('click', function(e) {
  if ( this.checked ) {
    jQuery('#quickcab_settings\\[autocomplete_country\\]').removeAttr('disabled');
  } else {
    jQuery('#quickcab_settings\\[autocomplete_country\\]').attr('disabled', 'disabled');
  }
});

jQuery('#quickcab_booking_form_enforce_autocomplete_restriction').on('click', function(e) {
  if ( this.checked ) {
    jQuery('#quickcab_booking_form_autocomplete_country').removeAttr('disabled');
  } else {
    jQuery('#quickcab_booking_form_autocomplete_country').attr('disabled', 'disabled');
  }
});

jQuery('#quickcab_booking_form_enable_extra_waiting_time').on('click', function(e) {
  if ( this.checked ) {
    jQuery('#quickcab_booking_form_extra_waiting_time_unit').removeAttr('disabled');
    jQuery('#quickcab_booking_form_extra_waiting_time_max_value').removeAttr('disabled');
  } else {
    jQuery('#quickcab_booking_form_extra_waiting_time_unit').attr('disabled', 'disabled');
    jQuery('#quickcab_booking_form_extra_waiting_time_max_value').attr('disabled', 'disabled');
  }
});

jQuery('#quickcab_booking_form_extra_waiting_time_unit').change(function() {
  if ( this.value == 'hours' ) {
    jQuery('.quickcab-booking-form-extra-time-unit').html(quickcab_js_settings.translations.hours);
  } else {
    jQuery('.quickcab-booking-form-extra-time-unit').html(quickcab_js_settings.translations.minutes);
  }
});

jQuery('#quickcab_vehicle_waiting_unit').change(function() {
  if ( this.value == 'hours' ) {
    jQuery('.quickcab-vehicle-extra-time-unit').html(quickcab_js_settings.translations.hour);
  } else {
    jQuery('.quickcab-vehicle-extra-time-unit').html(quickcab_js_settings.translations.minute);
  }
});

jQuery('body').on('click', '.quickcab-checkout-field-remove-option', function(e) {
  e.preventDefault();

  jQuery(this).closest('.quickcab-checkout-field-option-group').remove();
});

jQuery('body').on('click', '.quickcab-checkout-field-add-option', function(e) {
  e.preventDefault();

  var inputGroup = '<li class="quickcab-checkout-field-option-group">\
    <input name="quickcab_checkout_field_input_options[]" type="text" class="regular-text" id="quickcab_checkout_field_input_options">\
    <a href="#" class="button quickcab-checkout-field-remove-option">' + quickcab_js_settings.translations.remove + '</a>\
  </li>';

  jQuery('.quickcab-checkout-field-options-list').append(inputGroup);
});

jQuery('#quickcab_checkout_field_input_type').change(function() {
  if (this.value == 'text' || this.value == 'textarea') {
    jQuery('#quickcab_checkout_field_input_options_tbody').hide();
  } else {
    jQuery('#quickcab_checkout_field_input_options_tbody').show();
  }
});

if ( jQuery('#quickcab-calendar').length > 0 ) {
  var calendar = new FullCalendar.Calendar(document.getElementById('quickcab-calendar'), {
    plugins: ['dayGrid', 'timeGrid', 'list'],
    defaultView: quickcab_js_settings.default_calendar_period,
    firstDay: parseInt(quickcab_js_settings.fullcalendar_start_of_week),
    locale: quickcab_js_settings.fullcalendar_locale_code,
    fixedWeekCount: false,
    header: {
      center: 'title',
      right: 'today prev,next',
      left: 'dayGridMonth,dayGridWeek,timeGridDay,listWeek'
    },
    eventSources: [{
      url: quickcab_js_settings.ajax_url,
      method: 'POST',
      extraParams: {
        action: 'quickcab_get_calendar_events'
      },
      failure: function() {
        console.error('There was an error while fetching calendar orders.');
      }
    }],
    eventRender: function(event) {
      var $el = jQuery(event.el);

      $el.qtip({
        content: {
          text: event.event.extendedProps.description,
          title: event.event.extendedProps.qtipTitle
        },
        position: {
          viewport: jQuery('#wpbody'),
          my: 'bottom center',
          at: 'top center'
        },
        hide: {
          delay: 100,
          fixed: true
        },
        style: {
          classes: 'qtip-bootstrap quickcab-qtip'
        }
      });
    }
  });

  calendar.render();
}
