<?php

if ( !defined('ABSPATH') ) exit;

class QuickCab_Order {

  const METERS_TO_MILES = 0.00062137;
  const METERS_TO_KM = 0.001;

  // WP_Post fields
  public $ID = 0;
  public $post_author = 0;
  public $post_date = '0000-00-00 00:00:00';
  public $post_date_gmt = '0000-00-00 00:00:00';
  public $post_content = '';
  public $post_title = '';
  public $post_excerpt = '';
  public $post_status = 'publish';
  public $comment_status = 'open';
  public $ping_status = 'open';
  public $post_password = '';
  public $post_name = '';
  public $to_ping = '';
  public $pinged = '';
  public $post_modified = '0000-00-00 00:00:00';
  public $post_modified_gmt = '0000-00-00 00:00:00';
  public $post_content_filtered = '';
  public $post_parent = 0;
  public $guid = '';
  public $menu_order = 0;
  public $post_mime_type = '';
  public $comment_count = 0;
  public $filter;

  // journey details
  private $vehicle_id;
  private $starting_lat;
  private $starting_long;
  private $destination_lat;
  private $destination_long;
  private $starting_address;
  private $destination_address;
  private $waypoints;
  private $occupants;
  private $suitcases;
  private $is_return;
  private $departure_time;
  private $return_departure_time;
  private $duration_in_seconds;
  private $duration_in_words;
  private $duration_in_seconds_traffic;
  private $duration_in_words_traffic;
  private $return_duration_in_seconds;
  private $return_duration_in_words;
  private $return_duration_in_seconds_traffic;
  private $return_duration_in_words_traffic;
  private $distance_in_metres;
  private $distance_in_words;
  private $base_to_collection_time;
  private $destination_to_base_time;
  private $base_location;
  private $form_id;
  private $waiting_time;

  // order details
  private $total;
  private $subtotal;
  private $tax;
  private $tax_rate;
  private $currency;
  private $gateway;
  private $mode;
  private $key;
  private $customer_id;
  private $user_id;
  private $order_date;
  private $status;

  // customer details
  private $first_name;
  private $last_name;
  private $email;
  private $address1;
  private $address2;
  private $town_city;
  private $zip;
  private $country;

  private $wp_post;
  private $meta_prefix;

  public function __construct($id) {
    $this->meta_prefix = '_quickcab_order_';
    $order = WP_Post::get_instance($id);

    if ( $order ) {
      $this->setup_order($order);
    } else {
      return false;
    }
  }

  /**
   * Return the ID of the WP_Post object
   *
   * @since  1.0
   * @return int  order ID
   */
  public function get_ID() {
    return $this->ID;
  }

  /**
   * Return the vehicle id for the order
   *
   * @since  1.0
   * @return int  the vehicle id
   */
  public function get_vehicle_id() {
    return apply_filters('quickcab_order_get_vehicle_id', $this->vehicle_id, $this->get_ID());
  }

  /**
   * Return the post status for the order
   *
   * @since  1.0
   * @return string  the post status
   */
  public function get_post_status() {
    return apply_filters('quickcab_order_get_post_status', $this->post_status, $this->get_ID());
  }

  /**
   * Return the status for the order
   *
   * @since  1.0
   * @return string  the post status
   */
  public function get_status() {
    return apply_filters('quickcab_order_get_status', $this->status, $this->get_ID());
  }

  /**
   * Return the vehicle for the order
   *
   * @since  1.0
   * @return QuickCab_Vehicle  the vehicle
   */
  public function get_vehicle() {
    return new QuickCab_Vehicle($this->get_vehicle_id());
  }

  /**
   * Return the vehicle name for the order
   *
   * @since  1.0
   * @return string  the vehicle name
   */
  public function get_vehicle_name() {
    return $this->get_vehicle()->get_name();
  }

  public function get_form_id() {
    return apply_filters('quickcab_order_get_form_id', $this->form_id, $this->get_ID());
  }

  public function get_booking_form() {
    return new QuickCab_Booking_Form($this->get_form_id());
  }

  /**
   * Return the vehicle name for the order
   *
   * @since  1.0
   * @return string  the vehicle name
   */
  public function get_wc_order() {
    return wc_get_order($this->get_ID());
  }

  /**
   * Return the starting lat for the order
   *
   * @since  1.0
   * @return string  the starting lat
   */
  public function get_starting_lat() {
    return apply_filters('quickcab_order_get_starting_lat', $this->starting_lat, $this->get_ID());
  }

  /**
   * Return the starting long for the order
   *
   * @since  1.0
   * @return string  the starting long
   */
  public function get_starting_long() {
    return apply_filters('quickcab_order_get_starting_long', $this->starting_long, $this->get_ID());
  }

  /**
   * Return the destination lat for the order
   *
   * @since  1.0
   * @return string  the destination lat
   */
  public function get_destination_lat() {
    return apply_filters('quickcab_order_get_destination_lat', $this->destination_lat, $this->get_ID());
  }

  /**
   * Return the destination long
   *
   * @since  1.0
   * @return string  the destination long
   */
  public function get_destination_long() {
    return apply_filters('quickcab_order_get_destination_long', $this->destination_long, $this->get_ID());
  }

  /**
   * Return the starting address
   *
   * @since  1.0
   * @return string  the starting address
   */
  public function get_starting_address() {
    return apply_filters('quickcab_order_get_starting_address', $this->starting_address, $this->get_ID());
  }

  /**
   * Return the destination address
   *
   * @since  1.0
   * @return string  the destination address
   */
  public function get_destination_address() {
    return apply_filters('quickcab_order_get_destination_address', $this->destination_address, $this->get_ID());
  }

  /**
   * Return the occupants
   *
   * @since  1.0
   * @return int  the occupants
   */
  public function get_occupants() {
    return apply_filters('quickcab_order_get_occupants', $this->occupants, $this->get_ID());
  }

  /**
   * Return the suitcases
   *
   * @since  1.0
   * @return int  the suitcases
   */
  public function get_suitcases() {
    return apply_filters('quickcab_order_get_suitcases', $this->suitcases, $this->get_ID());
  }

  /**
   * Return the is_return field
   *
   * @since  1.0
   * @return boolean  is the journey a return journey
   */
  public function is_return() {
    return apply_filters('quickcab_order_get_is_return', $this->is_return, $this->get_ID());
  }

  /**
   * Return the order date
   *
   * @since   1.0
   * @return  string     the order date
   */
  public function get_order_date() {
    return apply_filters('quickcab_order_get_order_date', $this->order_date, $this->get_ID());
  }

  /**
   * Return the order date in the given format
   *
   * @since  1.0
   * @return string  the time of the departure time in the given format
   */
  public function get_order_date_in_format($format = 'F j, Y h:i a') {
    return date($format, strtotime($this->get_order_date()));
  }

  /**
   * Return the departure time
   *
   * @since  1.0
   * @return string  the departure time (in format: Y-m-d H:i:s)
   */
  public function get_departure_time() {
    return apply_filters('quickcab_order_get_departure_time', $this->departure_time, $this->get_ID());
  }

  /**
   * Return the departure timestamp
   *
   * @since  1.0
   * @return int  the departure timestamp
   */
  public function get_departure_timestamp() {
    return $this->get_departure_time_in_format('U');
  }

  /**
   * Return the departure datetime in the given format
   *
   * @since  1.0
   * @return string  the time of the departure time in the given format
   */
  public function get_departure_time_in_format($format = false) {
    if ( !$format ) {
      $format = get_option('date_format') . ' ' . get_option('time_format');
    }

    $timezone = new DateTimeZone(quickcab_get_wordpress_timezone());
    $departure_time = DateTime::createFromFormat('Y-m-d H:i:s', $this->get_departure_time(), $timezone);
    return $departure_time->format($format);
  }

  /**
   * Return the estimated arrival timestamp
   *
   * @since  1.0
   * @return int  the estimated arrival timestamp
   */
  public function get_estimated_arrival_timestamp() {
    $duration = empty($this->get_duration_in_seconds_traffic()) ? $this->get_duration_in_seconds() : $this->get_duration_in_seconds_traffic();
    return $this->get_departure_timestamp() + $duration;
  }

  /**
   * Return the estimated arrival time in the given format
   *
   * @since   1.0
   * @param   string     $format  the time format
   * @return  string              the formatted time
   */
  public function get_estimated_arrival_time_in_format($format = false) {
    if ( !$format ) {
      $format = get_option('date_format') . ' ' . get_option('time_format');
    }

    $timezone = new DateTimeZone(quickcab_get_wordpress_timezone());
    $departure_time = new DateTime('now', $timezone);
    $departure_time->setTimestamp($this->get_estimated_arrival_timestamp());

    return $departure_time->format($format);
  }

  /**
   * Return the return departure time
   *
   * @since  1.0
   * @return string  the return departure time (in format: Y-m-d H:i:s)
   */
  public function get_return_departure_time() {
    return apply_filters('quickcab_order_get_return_departure_time', $this->return_departure_time, $this->get_ID());
  }

  /**
   * Return the return departure timestamp
   *
   * @since  1.0
   * @return int  the return departure timestamp
   */
  public function get_return_departure_timestamp() {
    return $this->get_return_departure_time_in_format('U');
  }

  /**
   * Return the return departure datetime in the given format
   *
   * @since  1.0
   * @return string  the time of the return departure time in the given format
   */
  public function get_return_departure_time_in_format($format = false) {
    if ( !$format ) {
      $format = get_option('date_format') . ' ' . get_option('time_format');
    }

    $timezone = new DateTimeZone(quickcab_get_wordpress_timezone());
    $return_departure_time = DateTime::createFromFormat('Y-m-d H:i:s', $this->get_return_departure_time(), $timezone);
    return $return_departure_time->format($format);
  }

  /**
   * Return the estimated return arrival timestamp
   *
   * @since  1.0
   * @return int  the estimated return arrival timestamp
   */
  public function get_estimated_return_arrival_timestamp() {
    $duration = empty($this->get_return_duration_in_seconds_traffic()) ? $this->get_return_duration_in_seconds() : $this->get_return_duration_in_seconds_traffic();
    return $this->get_return_departure_timestamp() + $duration;
  }

  /**
   * Return the estimated return arrival time in the given format
   *
   * @since   1.0
   * @param   string     $format  the time format
   * @return  string              the formatted time
   */
  public function get_estimated_return_arrival_time_in_format($format = false) {
    if ( !$format ) {
      $format = get_option('date_format') . ' ' . get_option('time_format');
    }

    $timezone = new DateTimeZone(quickcab_get_wordpress_timezone());
    $departure_time = new DateTime('now', $timezone);
    $departure_time->setTimestamp($this->get_estimated_return_arrival_timestamp());

    return $departure_time->format($format);
  }

  /**
   * Return the duration in seconds
   *
   * @since  1.0
   * @return int  the duration in seconds
   */
  public function get_duration_in_seconds() {
    return apply_filters('quickcab_order_get_duration_in_seconds', $this->duration_in_seconds, $this->get_ID());
  }

  /**
   * Return the duration in minutes
   *
   * @since  1.0
   * @return int  the duration in minutes
   */
  public function get_duration_in_minutes() {
    return $this->get_duration_in_seconds() / 60;
  }

  /**
   * Return the duration in hours
   *
   * @since  1.0
   * @return int  the duration in hours
   */
  public function get_duration_in_hours() {
    return $this->get_duration_in_minutes() / 60;
  }

  /**
   * Return the return duration in seconds
   *
   * @since  1.0
   * @return int  the duration in seconds
   */
  public function get_return_duration_in_seconds() {
    return apply_filters('quickcab_order_get_return_duration_in_seconds', $this->return_duration_in_seconds, $this->get_ID());
  }

  /**
   * Return the return duration in minutes
   *
   * @since  1.0
   * @return int  the duration in minutes
   */
  public function get_return_duration_in_minutes() {
    return $this->get_return_duration_in_seconds() / 60;
  }

  /**
   * Return the return duration in hours
   *
   * @since  1.0
   * @return int  the duration in hours
   */
  public function get_return_duration_in_hours() {
    return $this->get_return_duration_in_minutes() / 60;
  }

  /**
   * Return the duration in seconds with traffic
   *
   * @since  1.0
   * @return int  the duration in seconds with traffic
   */
  public function get_duration_in_seconds_traffic() {
    return apply_filters('quickcab_order_get_duration_in_seconds_traffic', $this->duration_in_seconds_traffic, $this->get_ID());
  }

  /**
   * Return the duration in minutes with traffic
   *
   * @since  1.0
   * @return int  the duration in minutes with traffic
   */
  public function get_duration_in_minutes_traffic() {
    return $this->get_duration_in_seconds_traffic() / 60;
  }

  /**
   * Return the duration in hours with traffic
   *
   * @since  1.0
   * @return int  the duration in hours with traffic
   */
  public function get_duration_in_hours_traffic() {
    return $this->get_duration_in_minutes_traffic() / 60;
  }

  /**
   * Return the return duration in seconds with traffic
   *
   * @since  1.0
   * @return int  the duration in seconds with traffic
   */
  public function get_return_duration_in_seconds_traffic() {
    return apply_filters('quickcab_order_get_return_duration_in_seconds_traffic', $this->return_duration_in_seconds_traffic, $this->get_ID());
  }

  /**
   * Return the return duration in minutes with traffic
   *
   * @since  1.0
   * @return int  the duration in minutes with traffic
   */
  public function get_return_duration_in_minutes_traffic() {
    return $this->get_return_duration_in_seconds_traffic() / 60;
  }

  /**
   * Return the return duration in hours with traffic
   *
   * @since  1.0
   * @return int  the duration in hours with traffic
   */
  public function get_return_duration_in_hours_traffic() {
    return $this->get_return_duration_in_minutes_traffic() / 60;
  }

  /**
   * Return the duration in words with traffic
   *
   * @since  1.0
   * @return string  the duration in words with traffic
   */
  public function get_duration_in_words_traffic() {
    return apply_filters('quickcab_order_get_duration_in_words', $this->duration_in_words_traffic, $this->get_ID());
  }

  /**
   * Return the duration in words
   *
   * @since  1.0
   * @return string  the duration in words
   */
  public function get_duration_in_words() {
    return apply_filters('quickcab_order_get_duration_in_words', $this->duration_in_words, $this->get_ID());
  }

  /**
   * Return the duration in words with traffic
   *
   * @since  1.0
   * @return string  the duration in words with traffic
   */
  public function get_return_duration_in_words_traffic() {
    return apply_filters('quickcab_order_get_return_duration_in_words', $this->return_duration_in_words_traffic, $this->get_ID());
  }

  /**
   * Return the duration in words
   *
   * @since  1.0
   * @return string  the duration in words
   */
  public function get_return_duration_in_words() {
    return apply_filters('quickcab_order_get_return_duration_in_words', $this->return_duration_in_words, $this->get_ID());
  }

  /**
   * Return the distance in metres
   *
   * @since  1.0
   * @return int  the distance in metres
   */
  public function get_distance_in_metres() {
    return apply_filters('quickcab_order_get_distance_in_metres', $this->distance_in_metres, $this->get_ID());
  }

  /**
   * Return the distance the unit the user has selected in settings
   *
   * @since  1.0
   * @return int  the distance in give unit
   */
  public function get_distance_in_unit() {
    if ( quickcab_get_option('distance_units') == 'miles' ) {
      $distance = $this->distance_in_meters * self::METERS_TO_MILES;
    } else {
      $distance = $this->distance_in_meters * self::METERS_TO_KM;
    }

    return apply_filters('quickcab_order_get_distance_in_unit', $distance, $this->get_ID());
  }

  /**
   * Return the distance in words
   *
   * @since  1.0
   * @return string  the distance in words
   */
  public function get_distance_in_words() {
    return apply_filters('quickcab_order_get_distance_in_words', $this->distance_in_words, $this->get_ID());
  }

  /**
   * Return the base to collection address time in seconds
   *
   * @since  1.0
   * @return string  the base to collection address time in seconds
   */
  public function get_base_to_collection_time() {
    return apply_filters('quickcab_order_get_base_to_collection_time', $this->base_to_collection_time, $this->get_ID());
  }

  /**
   * Return the destination address to base time in seconds
   *
   * @since  1.0
   * @return string  the destination address to base time in seconds
   */
  public function get_destination_to_base_time() {
    return apply_filters('quickcab_order_get_destination_to_base_time', $this->destination_to_base_time, $this->get_ID());
  }

  /**
   * Return the base location address at the time of order
   *
   * @since  1.0
   * @return string  the base location
   */
  public function get_base_location() {
    return apply_filters('quickcab_order_get_base_location', $this->base_location, $this->get_ID());
  }

  /**
   * Return the waiting time
   *
   * @since  1.0
   * @return float  the waiting time
   */
  public function get_waiting_time() {
    return apply_filters('quickcab_order_get_waiting_time', $this->waiting_time, $this->get_ID());
  }

  public function get_formatted_waiting_time() {
    $booking_form = $this->get_booking_form();

    if ( $booking_form->get_extra_waiting_time_unit() == 'minutes' ) {
      return $this->get_waiting_time() . ' ' . __('minutes', 'quickcab');
    }

    return round($this->get_waiting_time() / 60, 2) . ' ' . __('hours', 'quickcab');
  }

  /**
   * Return the total of the order
   *
   * @since  1.0
   * @return float  the total for the order
   */
  public function get_total() {
    return apply_filters('quickcab_order_get_total', $this->total, $this->get_ID());
  }

  /**
   * Return the subtotal of the order
   *
   * @since  1.0
   * @return float  the subtotal for the order
   */
  public function get_subtotal() {
    return apply_filters('quickcab_order_get_subtotal', $this->subtotal, $this->get_ID());
  }

  /**
   * Return the gateway of the order
   *
   * @since  1.0
   * @return string  the gateway for the order
   */
  public function get_gateway() {
    return apply_filters('quickcab_order_get_gateway', $this->gateway, $this->get_ID());
  }

  /**
   * Return the tax of the order
   *
   * @since  1.0
   * @return float  the tax for the order
   */
  public function get_tax() {
    return apply_filters('quickcab_order_get_tax', $this->tax, $this->get_ID());
  }

  /**
   * Return the tax rate of the order
   *
   * @since  1.0
   * @return float  the tax rate for the order
   */
  public function get_tax_rate() {
    return apply_filters('quickcab_order_get_tax_rate', $this->tax_rate, $this->get_ID());
  }

  /**
   * Return the currency of the order
   *
   * @since  1.0
   * @return string  the currency for the order
   */
  public function get_currency() {
    return apply_filters('quickcab_order_get_currency', $this->currency, $this->get_ID());
  }

  /**
   * Return the mode of the order
   *
   * @since  1.0
   * @return string  the mode for the order
   */
  public function get_mode() {
    return apply_filters('quickcab_order_get_mode', $this->mode, $this->get_ID());
  }

  /**
   * Return the key of the order
   *
   * @since  1.0
   * @return string  the key for the order
   */
  public function get_key() {
    return apply_filters('quickcab_order_get_key', $this->key, $this->get_ID());
  }

  /**
   * Return the user id of the order
   *
   * @since  1.0
   * @return int  the user id for the order
   */
  public function get_user_id() {
    return apply_filters('quickcab_order_get_user_id', $this->user_id, $this->get_ID());
  }

  /**
   * Return the user of the order
   *
   * @since  1.0
   * @return [WP_User|false]  the user for the order
   */
  public function get_user() {
    return get_user_by('ID', $this->get_user_id());
  }

  /**
   * Return the date of the order in a custom format
   *
   * @since  1.0
   * @return string  the date for the order in a custom format
   */
  public function get_date($format = null) {
    return get_the_date($format, $this->get_ID());
    return date($format, strtotime($this->wp_post->post_date));
  }

  /**
   * Return the first name of the user who ordered
   *
   * @since  1.0
   * @return string  the first name of the user who ordered
   */
  public function get_first_name() {
    return apply_filters('quickcab_order_get_first_name', $this->first_name, $this->get_ID());
  }

  /**
   * Return the last name of the user who ordered
   *
   * @since  1.0
   * @return string  the last name of the user who ordered
   */
  public function get_last_name() {
    return apply_filters('quickcab_order_get_last_name', $this->last_name, $this->get_ID());
  }

  /**
   * Return the full name of user who ordered
   *
   * @since  1.0
   * @return string  the last name of the user who ordered
   */
  public function get_full_name() {
    return apply_filters('quickcab_order_get_full_name', $this->get_first_name() . ' ' . $this->get_last_name(), $this->get_ID());
  }

  /**
   * Return the email of user who ordered
   *
   * @since  1.0
   * @return string  the email of the user who ordered
   */
  public function get_email() {
    return apply_filters('quickcab_order_get_email', $this->email, $this->get_ID());
  }

  /**
   * Return the address line 1 of user who ordered
   *
   * @since  1.0
   * @return string  the address line 1 of the user who ordered
   */
  public function get_address1() {
    return apply_filters('quickcab_order_get_address1', $this->address1, $this->get_ID());
  }

  /**
   * Return the address line 2 of user who ordered
   *
   * @since  1.0
   * @return string  the address line 2 of the user who ordered
   */
  public function get_address2() {
    return apply_filters('quickcab_order_get_address2', $this->address2, $this->get_ID());
  }

  /**
   * Return the town/city of user who ordered
   *
   * @since  1.0
   * @return string  the town/city of the user who ordered
   */
  public function get_town_city() {
    return apply_filters('quickcab_order_get_town_city', $this->town_city, $this->get_ID());
  }

  /**
   * Return the zip of user who ordered
   *
   * @since  1.0
   * @return string  the zip of the user who ordered
   */
  public function get_zip() {
    return apply_filters('quickcab_order_get_zip', $this->zip, $this->get_ID());
  }

  /**
   * Return the country of user who ordered
   *
   * @since  1.0
   * @return string  the country of the user who ordered
   */
  public function get_country() {
    return apply_filters('quickcab_order_get_country', $this->country, $this->get_ID());
  }

  /**
   * Get the fully formatted address, lines separated by <br>
   *
   * @since  1.0
   * @return string  formatted address
   */
  public function get_full_address() {
    $address_fields = array('address1', 'address2', 'town_city', 'zip', 'country');
    $populated_fields = array();

    foreach ( $address_fields as $address_field ) {
      if ( !empty($this->$address_field) ) {
        array_push($populated_fields, $this->$address_field);
      }
    }

    return implode('<br>', $populated_fields);
  }

  /**
   * Return the price breakdown for the order
   *
   * @since  1.0
   * @return array  the list of different components that make the total price
   */
  public function get_price_breakdown() {
    return apply_filters('quickcab_order_get_price_breakdown', $this->price_breakdown, $this->get_ID());
  }

  /**
   * Get the title of the journey for the calendar
   *
   * @since  1.0
   * @return string  the title
   */
  public function get_calendar_title() {
    $title = $this->get_starting_address() . ' => ' . $this->get_destination_address() . ' - ' . $this->get_vehicle_name();
    return apply_filters('quickcab_order_get_calendar_title', $title, $this->get_ID());
  }

  /**
   * Get the title of the journey for the calendar
   *
   * @since  1.0
   * @return string  the title
   */
  public function get_calendar_return_journey_title() {
    $title = $this->get_destination_address() . ' => ' . $this->get_starting_address() . ' - ' . $this->get_vehicle_name();
    return apply_filters('quickcab_order_get_calendar_return_journey_title', $title, $this->get_ID());
  }

  public function get_waypoints() {
    return apply_filters('quickcab_order_get_waypoints', $this->waypoints, $this->get_ID());
  }

  public function get_static_map_url() {
    $maps_url = "https://maps.googleapis.com/maps/api/staticmap";
    $maps_url .= "?size=500x400";
    $maps_url .= "&markers=color:green%7Clabel:1%7C" . $this->get_starting_lat() . "," . $this->get_starting_long();

    $waypoints = $this->get_waypoints();
    for ( $i = 0; $i < count($waypoints); $i++ ) {
      $waypoint = $waypoints[$i];
      $maps_url .= "&markers=color:green%7Clabel:" . ($i + 2) . "%7C" . $waypoint['lat'] . "," . $waypoint['lng'];
    }

    $maps_url .= "&markers=color:red%7Clabel:" . count($waypoints) . "%7C" . $this->get_destination_lat() . "," . $this->get_destination_long();
    $maps_url .= "&key=" . quickcab_get_option('google_static_maps_api_key');

    return apply_filters('quickcab_order_get_static_map_url', $maps_url, $this->get_ID());
  }

  /**
   * Get the meta prefix for orders
   *
   * @since   1.0
   * @return  string     meta prefix
   */
  public function get_meta_prefix() {
    return $this->meta_prefix;
  }

  public function get_custom_checkout_fields() {
    $checkout_field_keys = $this->get_custom_checkout_field_keys();

    $custom_fields = array();

    foreach ( $checkout_field_keys as $checkout_field_key ) {
      $field = array(
        'label' => $this->get_meta('custom_checkout_field_label_' . $checkout_field_key),
        'value' => $this->get_meta('custom_checkout_field_' . $checkout_field_key),
        'show_in_email' => (bool) $this->get_meta('custom_checkout_field_show_email_' . $checkout_field_key)
      );

      array_push($custom_fields, $field);
    }

    return $custom_fields;
  }

  public function get_custom_checkout_field_keys() {
    return apply_filters('quickcab_order_get_custom_checkout_field_keys', $this->custom_checkout_field_keys, $this->get_ID());
  }

  /**
   * Returns array of QuickCab_Orders that have the meta key/value passed as parameters
   *
   * @since   1.0
   * @param   string     $key    the meta key
   * @param   string     $value  the meta value
   * @return  [QuickCab_Order]         the QuickCab_Orders that satisfy the key/value passed
   */
  public static function find_by_meta_key($key, $value) {
    global $wpdb;

    $results = $wpdb->get_results($wpdb->prepare(
      "SELECT post_id FROM " . $wpdb->postmeta . " WHERE meta_key = '_quickcab_order_" . esc_sql($key) . "' AND meta_value = %s",
      $value
    ));

    $orders = array_map(function($row) {
      return new QuickCab_Order($row->post_id);
    }, $results);

    return array_filter($orders, function($order) {
      return $order->get_ID() !== 0;
    });
  }

  /**
   * Get the meta data for the order
   *
   * @since  1.0
   * @param  string  $key     the key to fetch
   * @param  boolean $single  get single data or array
   * @return [array|string]   value of meta data if single = true, array if not
   */
  public function get_meta($key = '', $single = true) {
    $value = get_post_meta($this->ID, $this->meta_prefix . $key, $single);
    return apply_filters('quickcab_order_get_meta_' . $key, $value, $this->ID, $single);
  }

  /**
   * Update the meta data for the order
   *
   * @since  1.0
   * @param  string $key         the key to update
   * @param  string $value       the new value
   * @return [int|bool]          meta ID if key didn't exist, true on success, false on fail
   */
  public function update_meta($key = '', $value = '') {
    if ( empty($key) || $key == '' ) {
      return false;
    }

    $meta_key = $this->meta_prefix . $key;
    $value = apply_filters('quickcab_order_update_meta_' . $meta_key, $value, $this->get_ID());
    $updated = update_post_meta($this->get_ID(), $meta_key, $value);

    if ( $updated ) {
      $this->$key = $value;
    }

    return $updated;
  }

  /**
   * Delete the meta data for the order
   *
   * @since  1.0
   * @param  string $key  the key to delete
   * @return bool         success or not
   */
  public function delete_meta($key) {
    if ( empty($key) || $key == '' ) {
      return false;
    }

    $key = $this->meta_prefix . $key;

    return delete_post_meta($this->get_ID(), $key);
  }

  /**
   * Populates the default WordPress Post fields
   *
   * @since  1.0
   * @param  WP_Post $post  the WP_Post
   * @return bool           if the save was successful
   */
  private function populate_post_fields($post) {
    if ( !is_object($post) || !is_a($post, 'WP_Post') || $post->post_type !== 'quickcab_order' ) {
      return false;
    }

    foreach ( $post as $key => $value ) {
      $this->$key = $value;
    }

    return true;
  }

  /**
   * Gets all QuickCab_Vehicles
   *
   * @since  1.0
   * @return array  array of all vehicles
   */
  public static function find_all() {
    $orders = array();

    $post_search = new WP_Query(array(
      'post_type' => 'quickcab_order',
      'post_staus' => array('publish'),
      'posts_per_page' => -1
    ));

    $unacceptable_status = apply_filters('quickcab_order_unacceptable_statuses', array(
      'auto-draft', 'future', 'draft', 'private', 'trash'
    ));

    foreach ( $post_search->posts as $order ) {
      if ( in_array($order->post_status, $unacceptable_status) ) {
        continue;
      }

      $orders[$order->ID] = new QuickCab_Order($order->ID);
    }

    return $orders;
  }

  /**
   * Setup the object variables
   *
   * @since  1.0
   * @param  WP_Post $order  the order WP_Post
   * @return bool            did the setup suceed?
   */
  private function setup_order($order = null) {
    if ( is_null($order) || !is_a($order, 'WP_Post') || $order->post_type !== 'shop_order' ) {
      return false;
    }

    do_action('quickcab_before_setup_order', $this, $order);

    $this->ID = $order->ID;
    $this->wp_post = $order;

    $this->vehicle_id                         = $this->setup_vehicle_id();
    $this->total                              = $this->setup_total();
    $this->subtotal                           = $this->setup_subtotal();
    $this->tax                                = $this->setup_tax();
    $this->tax_rate                           = $this->setup_tax_rate();
    $this->currency                           = $this->setup_currency();
    $this->gateway                            = $this->setup_gateway();
    $this->mode                               = $this->setup_mode();
    $this->key                                = $this->setup_key();
    $this->customer_id                        = $this->setup_customer_id();
    $this->user_id                            = $this->setup_user_id();
    $this->first_name                         = $this->setup_first_name();
    $this->last_name                          = $this->setup_last_name();
    $this->email                              = $this->setup_email();
    $this->address1                           = $this->setup_address1();
    $this->address2                           = $this->setup_address2();
    $this->town_city                          = $this->setup_town_city();
    $this->zip                                = $this->setup_zip();
    $this->country                            = $this->setup_country();
    $this->starting_lat                       = $this->setup_starting_lat();
    $this->starting_long                      = $this->setup_starting_long();
    $this->destination_lat                    = $this->setup_destination_lat();
    $this->destination_long                   = $this->setup_destination_long();
    $this->starting_address                   = $this->setup_starting_address();
    $this->destination_address                = $this->setup_destination_address();
    $this->occupants                          = $this->setup_occupants();
    $this->suitcases                          = $this->setup_suitcases();
    $this->is_return                          = $this->setup_is_return();
    $this->departure_time                     = $this->setup_departure_time();
    $this->return_departure_time              = $this->setup_return_departure_time();
    $this->duration_in_seconds                = $this->setup_duration_in_seconds();
    $this->duration_in_words                  = $this->setup_duration_in_words();
    $this->duration_in_seconds_traffic        = $this->setup_duration_in_seconds_traffic();
    $this->duration_in_words_traffic          = $this->setup_duration_in_words_traffic();
    $this->return_duration_in_seconds         = $this->setup_return_duration_in_seconds();
    $this->return_duration_in_words           = $this->setup_return_duration_in_words();
    $this->return_duration_in_seconds_traffic = $this->setup_return_duration_in_seconds_traffic();
    $this->return_duration_in_words_traffic   = $this->setup_return_duration_in_words_traffic();
    $this->distance_in_metres                 = $this->setup_distance_in_metres();
    $this->distance_in_words                  = $this->setup_distance_in_words();
    $this->price_breakdown                    = $this->setup_price_breakdown();
    $this->order_date                         = $this->setup_order_date();
    $this->status                             = $this->setup_status();
    $this->custom_checkout_field_keys         = $this->setup_custom_checkout_field_keys();
    $this->base_to_collection_time            = $this->setup_base_to_collection_time();
    $this->destination_to_base_time           = $this->setup_destination_to_base_time();
    $this->base_location                      = $this->setup_base_location();
    $this->form_id                            = $this->setup_form_id();
    $this->waiting_time                       = $this->setup_waiting_time();
    $this->waypoints                          = $this->setup_waypoints();

    $this->populate_post_fields($this->wp_post);

    do_action('quickcab_after_setup_order', $this, $order);

    return true;
  }

  /**
   * Setup the vehicle id for the order
   *
   * @since  1.0
   * @return int  the vehicle id
   */
  private function setup_vehicle_id() {
    return (int) $this->get_meta('vehicle_id');
  }

  /**
   * Setup the total for the order
   *
   * @since  1.0
   * @return float  the total
   */
  private function setup_total() {
    return (float) $this->get_meta('total');
  }

  /**
   * Setup the subtotal for the order
   *
   * @since  1.0
   * @return float  the subtotal
   */
  private function setup_subtotal() {
    return (float) $this->get_meta('subtotal');
  }

  /**
   * Setup the tax for the order
   *
   * @since  1.0
   * @return float  the tax
   */
  private function setup_tax() {
    return (float) $this->get_meta('tax');
  }

  /**
   * Setup the tax rate for the order
   *
   * @since  1.0
   * @return float  the tax rate
   */
  private function setup_tax_rate() {
    return (float) $this->get_meta('tax_rate');
  }

  /**
   * Setup the currency for the order
   *
   * @since  1.0
   * @return string  the currency
   */
  private function setup_currency() {
    return $this->get_meta('currency');
  }

  /**
   * Setup the gateway for the order
   *
   * @since  1.0
   * @return string  the gateway
   */
  private function setup_gateway() {
    return $this->get_meta('gateway');
  }

  /**
   * Setup the mode for the order
   *
   * @since  1.0
   * @return string  the mode
   */
  private function setup_mode() {
    return $this->get_meta('mode');
  }

  /**
   * Setup the key for the order
   *
   * @since  1.0
   * @return string  the key
   */
  private function setup_key() {
    return $this->get_meta('key');
  }

  /**
   * Setup the customer ID for the order
   *
   * @since  1.0
   * @return int  the customer id
   */
  private function setup_customer_id() {
    return (int) $this->get_meta('customer_id');
  }

  /**
   * Setup the user ID for the order
   *
   * @since  1.0
   * @return int  the user id
   */
  private function setup_user_id() {
    return (int) $this->get_meta('user_id');
  }

  /**
   * Setup the first name for the order
   *
   * @since  1.0
   * @return string  the first name
   */
  private function setup_first_name() {
    return $this->get_meta('first_name');
  }

  /**
   * Setup the last name for the order
   *
   * @since  1.0
   * @return string  the last name
   */
  private function setup_last_name() {
    return $this->get_meta('last_name');
  }

  /**
   * Setup the email for the order
   *
   * @since  1.0
   * @return string  the email
   */
  private function setup_email() {
    return $this->get_meta('email');
  }

  /**
   * Setup the address line 1 for the order
   *
   * @since  1.0
   * @return string  the address line 1
   */
  private function setup_address1() {
    return $this->get_meta('address1');
  }

  /**
   * Setup the address line 2 for the order
   *
   * @since  1.0
   * @return string  the address line 2
   */
  private function setup_address2() {
    return $this->get_meta('address2');
  }

  /**
   * Setup the town/city for the order
   *
   * @since  1.0
   * @return string  the town/city
   */
  private function setup_town_city() {
    return $this->get_meta('town_city');
  }

  /**
   * Setup the zip for the order
   *
   * @since  1.0
   * @return string  the zip
   */
  private function setup_zip() {
    return $this->get_meta('zip');
  }

  /**
   * Setup the country for the order
   *
   * @since  1.0
   * @return string  the country
   */
  private function setup_country() {
    return $this->get_meta('country');
  }

  /**
   * Setup the starting lat
   *
   * @since  1.0
   * @return string  the starting lat (in string format because floats would round)
   */
  private function setup_starting_lat() {
    return $this->get_meta('starting_lat');
  }

  /**
   * Setup the starting long
   *
   * @since  1.0
   * @return string  the starting long (in string format because floats would round)
   */
  private function setup_starting_long() {
    return $this->get_meta('starting_long');
  }

  /**
   * Setup the destination lat
   *
   * @since  1.0
   * @return string  the destination lat (in string format because floats would round)
   */
  private function setup_destination_lat() {
    return $this->get_meta('destination_lat');
  }

  /**
   * Setup the destination long
   *
   * @since  1.0
   * @return string  the destination long (in string format because floats would round)
   */
  private function setup_destination_long() {
    return $this->get_meta('destination_long');
  }

  /**
   * Setup the starting address
   *
   * @since  1.0
   * @return string  the starting address
   */
  private function setup_starting_address() {
    return $this->get_meta('starting_address');
  }

  /**
   * Setup the destination address
   *
   * @since  1.0
   * @return string  the destination address
   */
  private function setup_destination_address() {
    return $this->get_meta('destination_address');
  }

  /**
   * Setup the occupants
   *
   * @since  1.0
   * @return int  the occupants
   */
  private function setup_occupants() {
    return (int) $this->get_meta('occupants');
  }

  /**
   * Setup the suitcases
   *
   * @since  1.0
   * @return int  the suitcases
   */
  private function setup_suitcases() {
    return (int) $this->get_meta('suitcases');
  }

  /**
   * Setup the return
   *
   * @since  1.0
   * @return boolean  is the order a return journey
   */
  private function setup_is_return() {
    return (boolean) $this->get_meta('is_return');
  }

  /**
   * Setup the departure timestamp
   *
   * @since  1.0
   * @return string  the departure timestamp
   */
  private function setup_departure_time() {
    return $this->get_meta('departure_time');
  }

  /**
   * Setup the return departure timestamp
   *
   * @since  1.0
   * @return string  the return departure timestamp
   */
  private function setup_return_departure_time() {
    return $this->get_meta('return_departure_time');
  }

  /**
   * Setup the duration in seconds
   *
   * @since  1.0
   * @return int  the duration in seconds of the journey
   */
  private function setup_duration_in_seconds() {
    return (int) $this->get_meta('duration_in_seconds');
  }

  /**
   * Setup the duration in words
   *
   * @since  1.0
   * @return string  the duration in words
   */
  private function setup_duration_in_words() {
    return $this->get_meta('duration_in_words');
  }

  /**
   * Setup the duration in seconds (with traffic)
   *
   * @since  1.0
   * @return int  the duration in seconds
   */
  private function setup_duration_in_seconds_traffic() {
    return $this->get_meta('duration_in_seconds_traffic');
  }

  /**
   * Setup the duration in words (with traffic)
   *
   * @since  1.0
   * @return string  the duration in words
   */
  private function setup_duration_in_words_traffic() {
    return $this->get_meta('duration_in_words_traffic');
  }

  /**
   * Setup the return duration in seconds
   *
   * @since  1.0
   * @return int  the return duration in seconds of the journey
   */
  private function setup_return_duration_in_seconds() {
    return (int) $this->get_meta('return_duration_in_seconds');
  }

  /**
   * Setup the return duration in words
   *
   * @since  1.0
   * @return string  the return duration in words
   */
  private function setup_return_duration_in_words() {
    return $this->get_meta('return_duration_in_words');
  }

  /**
   * Setup the return duration in seconds (with traffic)
   *
   * @since  1.0
   * @return int  the return duration in seconds
   */
  private function setup_return_duration_in_seconds_traffic() {
    return $this->get_meta('return_duration_in_seconds_traffic');
  }

  /**
   * Setup the return duration in words (with traffic)
   *
   * @since  1.0
   * @return string  the return duration in words
   */
  private function setup_return_duration_in_words_traffic() {
    return $this->get_meta('return_duration_in_words_traffic');
  }

  /**
   * Setup the distance in metres
   *
   * @since  1.0
   * @return int  the distance in metres
   */
  private function setup_distance_in_metres() {
    return (int) $this->get_meta('distance_in_metres');
  }

  /**
   * Setup the distance in words
   *
   * @since  1.0
   * @return string  the distance in words
   */
  private function setup_distance_in_words() {
    return $this->get_meta('distance_in_words');
  }

  /**
   * Setup the price breakdown
   *
   * @since  1.0
   * @return array  the price breakdown
   */
  private function setup_price_breakdown() {
    return unserialize($this->get_meta('price_breakdown'));
  }

  /**
   * Setup the order date
   *
   * @since   1.0
   * @return  string     the datetime of the order
   */
  private function setup_order_date() {
    return $this->get_meta('order_date');
  }

  /**
   * Setup the order status
   *
   * @since   1.0
   * @return  string     the order status
   */
  private function setup_status() {
    return $this->get_meta('status');
  }

  /**
   * Setup the custom checkout fields
   *
   * @since   1.0
   * @return  string     the custom checkout fields
   */
  private function setup_custom_checkout_field_keys() {
    return unserialize($this->get_meta('custom_checkout_field_keys'));
  }

  /**
   * Setup the base to collection time
   *
   * @since   1.0
   * @return  int     the base to collection address time in seconds
   */
  private function setup_base_to_collection_time() {
    return intval($this->get_meta('base_to_collection_time'));
  }

  /**
   * Setup the destination to base time
   *
   * @since   1.0
   * @return  int     the destination address to base time in seconds
   */
  private function setup_destination_to_base_time() {
    return intval($this->get_meta('destination_to_base_time'));
  }

  /**
   * Setup the base location
   *
   * @since   1.0
   * @return  string     the base location address
   */
  private function setup_base_location() {
    return $this->get_meta('base_location');
  }

  /**
   * Setup the form ID
   *
   * @since   1.0
   * @return  string     the form ID
   */
  private function setup_form_id() {
    return intval($this->get_meta('form_id'));
  }

  /**
   * Setup the waiting time
   *
   * @since   1.0
   * @return  string     the form ID
   */
  private function setup_waiting_time() {
    return intval($this->get_meta('waiting_time'));
  }

  private function setup_waypoints() {
    $waypoints = array();
    $waypoints_count = intval($this->get_meta('waypoints_count'));

    for ( $i = 0; $i < $waypoints_count; $i++ ) {
      array_push($waypoints, array(
        'lat' => $this->get_meta('waypoint_' . $i . '_lat'),
        'lng' => $this->get_meta('waypoint_' . $i . '_lng'),
        'address' => $this->get_meta('waypoint_' . $i . '_address')
      ));
    }

    return array_filter($waypoints, function($waypoint) {
      return $waypoint['lat'] && $waypoint['lng'] && $waypoint['address'];
    });
  }
}
